# Experiment 1B
# 0 Preliminaries
library(nlme)
library(lattice)
set.seed(1234321)
library(gdata)
library(gtools)
library(gplots)

# 1 Input
data=read.table("pi2.dat", header=TRUE, dec=".")
gdata <- groupedData(logrt ~ 1 | vp, data=data, order.groups = F)


model1 <- lme(logrt ~ objw+repet+repside+lag, random = pdDiag(~ objw+repet+repside+lag), 
          data = gdata, method="ML")

# add the 4 meaningful 2-way interactions as fixed effects
model2a <- update(model1, logrt ~ objw+repet+repside+lag+objw*repet)
model2b <- update(model1, logrt ~ objw+repet+repside+lag+objw*repside)
model2c <- update(model1, logrt ~ objw+repet+repside+lag+objw*lag)
model2d <- update(model1, logrt ~ objw+repet+repside+lag+repside*lag)
anova(model1, model2a)
anova(model1, model2b)
anova(model1, model2c)
anova(model1, model2d)
#2a and 2c better than 1 --> combine
model3 <- update(model2a, logrt ~ objw+repet+repside+lag+objw*repet+objw*lag)

# add random effects 
model3a <- update(model3, random = pdDiag(~ objw+repet+repside+lag+objw*repet+objw*lag))
anova(model3, model3a)
#no improvement

## now follow through the most successful model, including higher-order interactions
model4 <- update(model3, logrt ~ objw+repet+repside+lag+
                                 objw*repet + objw*repside + objw*lag + objw*repside*lag)
anova(model3, model4)

model4a <- update(model4, random = pdDiag(~ objw+repet+repside+lag + 
                                            objw*repet + objw*repside + objw*lag + objw*repside*lag))
anova(model3, model4, model4a)
# model3 still best on BIC

## now take best model so far and drop random effects 
model5a <- update(model3, random = pdDiag(~ objw+repet+repside))
model5b <- update(model3, random = pdDiag(~ objw+repet+lag))
model5c <- update(model3, random = pdDiag(~ objw+repside+lag))
model5d <- update(model3, random = pdDiag(~ repet+repside+lag))
anova(model3, model5a)
anova(model3, model5b)
anova(model3, model5c)
anova(model3, model5d)
# 5c no sign loss of fit --> retain 5c

## finally, try to remove fixed effects to test their significance 
model6a <- update(model5c, fixed = ~ objw+repet+repside+lag+objw*repet)
model6b <- update(model5c, fixed = ~ objw+repet+repside+lag+objw*lag)
model6c <- update(model5c, fixed = ~ objw+repet+repside+objw*repet)
model6d <- update(model5c, fixed = ~ objw+repet+lag+objw*repet+objw*lag)
model6e <- update(model5c, fixed = ~ objw+repside+lag+objw*lag)
model6f <- update(model5c, fixed = ~ repet+repside+lag)
#lme does not allow to remove main effects while including the interaction!

anova(model5c,model6a)
anova(model5c,model6b)
anova(model5c,model6c)
anova(model5c,model6d)
anova(model5c,model6e)
anova(model5c,model6f)

# Check for contribution of updating step 
model7 <- update(model5c, fixed = ~ objw+repet+repside+lag+upd+
                                     objw*repet+objw*lag)
anova(model5c, model7)
#significant improvement!
model8 <- update(model7, random = pdDiag(~ objw+repside+lag+upd))
anova(model7, model8)
#significant improvement!
#remove fixed effect of upd again
model9 <- update(model8, fixed = ~ objw+repet+repside+lag+
                                     objw*repet+objw*lag)
anova(model8, model9)
#yet another improvement!


model8a <- update(model8, fixed = ~ objw+repet+repside+lag+upd+objw*repet)
model8b <- update(model8, fixed = ~ objw+repet+repside+lag+upd+objw*lag)
model8c <- update(model8, fixed = ~ objw+repet+repside+objw*repet+objw*lag)
model8d <- update(model8, fixed = ~ objw+repet+repside+upd+objw*repet)
model8e <- update(model8, fixed = ~ objw+repet+lag+objw*repet+upd+objw*lag)
model8f <- update(model8, fixed = ~ objw+repside+lag+upd+objw*lag)
model8g <- update(model8, fixed = ~ repet+repside+lag+upd)
anova(model8, model8a)
anova(model8, model8b)
anova(model8, model8c)
anova(model8, model8d)
anova(model8, model8e)
anova(model8, model8f)
anova(model8, model8g)

source("Rsquared.R")
R <- Rsquared(model5c, gdata$logrt)
R
intervals(model5c)

save(model5c, file = "pi2model5c.rda")



# Visualization

load("pi2model5c.rda")
gdata$logpred <- predict(model5c,level=1)
gdata$rt <- exp(gdata$logrt)
gdata$pred <- exp(gdata$logpred)
write.table(gdata, file = "pi2.predobs.dat", append = FALSE, quote = FALSE, sep = " ")
### writes out for SPSS --> aggregate by conditions and read again!

aggrdata=read.table("pi2.aggr.dat", header=TRUE, dec=".")
aggrdata$condition <- 1000*(aggrdata$objw+1) + 100*(aggrdata$repet+1) + 10*(aggrdata$repside+2) + (aggrdata$lag+1)
#objw 1 = no, 2 = yes / repet: 1 = new, 2 = repeated / repside: 1 = same side, 2 = new, 3 = other side 
rdata <- reshape(aggrdata, idvar="vp", timevar="condition", drop=c("objw","repet","repside","lag","logrt","logpred"), direction="wide")
noswi.new <- rdata$rt.1121   #non-repeated items
noswi.ss.obs <- rdata[,c(4,6,8,10)]
noswi.ss.pred <- rdata[,c(5,7,9,11)]
noswi.os.obs <- rdata[,c(14,16,18)]
noswi.os.pred <- rdata[,c(15,17,19)]
swi.new <- rdata$rt.2121 
swi.ss.obs <- rdata[,c(22,24,36)]
swi.ss.pred <- rdata[,c(23,25,37)]
swi.os.obs <- rdata[,c(28,30,32,34)]
swi.os.pred <- rdata[,c(29,31,33,35)]

#Bakeman-McArthur correction
noswi.mean <- rowMeans(cbind(noswi.ss.obs, noswi.os.obs), na.rm=T)
noswi.ss.obs <- noswi.ss.obs - noswi.mean + mean(noswi.mean, na.rm=T)
noswi.os.obs <- noswi.os.obs - noswi.mean + mean(noswi.mean, na.rm=T)
swi.mean <- rowMeans(cbind(swi.ss.obs, swi.os.obs),na.rm=T)
swi.ss.obs <- swi.ss.obs - swi.mean + mean(swi.mean, na.rm=T)
swi.os.obs <- swi.os.obs - swi.mean + mean(swi.mean, na.rm=T)

source("plot.confint.R")
pt = c(1,0,17,0)
# point styles
par(font.main=1)
par(cex = 1.2)
#increases all text --> good for PowerPoint
cl <- c("black", "black", "black", "black")
#for print
#cl <- c("black", "red", "black", "blue")
#for PowerPoint
ltype <- c(1,1,2,2)
names <- c("1", "2", "3", "4+")

par(mfrow=c(2,1))
par(cex=0.7)
par(mar=c(4.5,4,3,3))
# bottom, left, top, right margins (in lines)

### no switch

x11()
plot.confint(na.omit(noswi.ss.obs), c(1:4), c(1:4), 0.05, xlim = c(0.5,4.5), ylim=c(0.5,3), cex=1.5,
             pch=pt[1], lty=ltype[1], xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot(c(1:4), colMeans(na.omit(noswi.ss.pred)), xlim=c(0.5,4.5), ylim=c(0.5,3), type = "l", 
             pch=pt[2], lty=ltype[2], cex=2, lwd=3, xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot.confint(noswi.os.obs, c(1:3), c(1,3,4), -0.05, xlim = c(0.5,4.5), ylim=c(0.5,3), cex=1.5,
             pch=pt[3], lty=ltype[3], xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot(c(1,3,4), colMeans(noswi.os.pred), xlim=c(0.5,4.5), ylim=c(0.5,3), type = "l", 
             pch=pt[4], lty=ltype[4], cex=2, lwd=3, xlab="Lag", ylab="RT (s)", axes=F)

axis(side=1, at=c(1:4), labels=names)
axis(2) #brings back y-axis
box()   #brings back the frame
legend(0.5, 2.4, legend=c("same side", "other side"), 
        col=cl[c(1,3)], lty=ltype[c(1,3)], pch=pt[c(1,3)], xjust=0, yjust=0)
polygon(x=c(0,5), y=c(mean(noswi.new),mean(noswi.new)))
title(main="No location switch")

### location switch

x11()
plot.confint(swi.ss.obs, c(1:2), c(1,3), 0.05, xlim = c(0.5,4.5), ylim=c(0.5,3), cex=1.5,
             pch=pt[1], lty=ltype[1], xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot(c(1,3), colMeans(swi.ss.pred[,1:2]), xlim=c(0.5,4.5), ylim=c(0.5,3), type = "l", 
             pch=pt[2], lty=ltype[2], lwd=3, cex=2, xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot.confint(swi.os.obs, c(1:4), c(1:4), -0.05, xlim = c(0.5,4.5), ylim=c(0.5,3), cex=1.5,
             pch=pt[3], lty=ltype[3], xlab="Lag", ylab="RT (s)", axes=F)
### not enough participants with valid cases for lag 4+!
par(new=T)
plot(c(1:4), colMeans(swi.os.pred), xlim=c(0.5,4.5), ylim=c(0.5,3), type = "l", 
             pch=pt[4], lty=ltype[4], lwd=3, cex=2, xlab="Lag", ylab="RT (s)", axes=F)

axis(side=1, at=c(1:4), labels=names)
axis(2) #brings back y-axis
box()   #brings back the frame
legend(3, 0.5, legend=c("same side", "other side"), 
        col=cl[c(1,3)], lty=ltype[c(1,3)], pch=pt[c(1,3)], xjust=0, yjust=0)
polygon(x=c(0,5), y=c(mean(swi.new),mean(swi.new)))
title(main="Location switch")
