# Experiment 2, Positions
# 0 Preliminaries
library(nlme)
library(lattice)
set.seed(1234321)    
library(gdata)
library(gtools)
library(gplots)

# 1 Input
data=read.table("pi3.pos.dat", header=TRUE, dec=".")
gdata <- groupedData(logrt ~ 1 | vp, data=data, order.groups = F)

model1 <- lme(logrt ~ setsize+objw+repet+repside+lag, random = pdDiag(~ setsize+objw+repet+repside+lag), 
          data = gdata, method="ML")

model2a <- update(model1, logrt ~ setsize+objw+repet+repside+lag+setsize*objw)
model2b <- update(model1, logrt ~ setsize+objw+repet+repside+lag+setsize*repet)
model2c <- update(model1, logrt ~ setsize+objw+repet+repside+lag+setsize*repside)
model2d <- update(model1, logrt ~ setsize+objw+repet+repside+lag+setsize*lag)
model2e <- update(model1, logrt ~ setsize+objw+repet+repside+lag+objw*repet)
model2f <- update(model1, logrt ~ setsize+objw+repet+repside+lag+objw*repside)
model2g <- update(model1, logrt ~ setsize+objw+repet+repside+lag+objw*lag)
model2h <- update(model1, logrt ~ setsize+objw+repet+repside+lag+repside*lag)

model3a <- update(model2a, random = pdDiag(~ setsize+objw+repet+repside+lag+setsize*objw))
#no convergence
model3b <- update(model2b, random = pdDiag(~ setsize+objw+repet+repside+lag+setsize*repet))
model3c <- update(model2c, random = pdDiag(~ setsize+objw+repet+repside+lag+setsize*repside))
model3d <- update(model2d, random = pdDiag(~ setsize+objw+repet+repside+lag+setsize*lag))
model3e <- update(model2e, random = pdDiag(~ setsize+objw+repet+repside+lag+objw*repet))
model3f <- update(model2f, random = pdDiag(~ setsize+objw+repet+repside+lag+objw*repside))
model3g <- update(model2g, random = pdDiag(~ setsize+objw+repet+repside+lag+objw*lag))
model3h <- update(model2h, random = pdDiag(~ setsize+objw+repet+repside+lag+repside*lag))

anova(model1, model2a)
anova(model1, model2b)
anova(model1, model2c)
anova(model1, model2d)
anova(model1, model2e)
anova(model1, model2f)
anova(model1, model2g)
anova(model1, model2h)
## improvement (on all 3 fit indices) for a, c, e, g, h

#anova(model2a, model3a)
anova(model2b, model3b)
anova(model2c, model3c)
anova(model2d, model3d)
anova(model2e, model3e)
anova(model2g, model3g)
anova(model2h, model3h)
# very slight improvement for g, h (none for others)
#--> best combination so far:
model4 <- update(model3h, logrt ~ setsize+objw+repet+repside+lag+
                                  setsize*objw + setsize*repside + objw*repet + objw*lag + repside*lag, 
		    random = pdDiag(~ setsize+objw+repet+repside+lag+
                                  objw*lag + repside*lag))

## now follow through the most successful model, including three-way interactions (some of which enforce inclusion of 2-way interactions that are not included in model4!)
model5a <- update(model4, logrt ~ setsize+objw+repet+repside+lag+
                                  setsize*objw + setsize*repet + setsize*repside + objw*repet + objw*lag + repside*lag +
					   setsize*objw*repet)
model5b <- update(model4, logrt ~ setsize+objw+repet+repside+lag+
                                  setsize*objw + setsize*repside + objw*repet + objw*repet + objw*lag + repside*lag +
					   setsize*objw*repside)
model5c <- update(model4, logrt ~ setsize+objw+repet+repside+lag+
                                  setsize*objw + setsize*repside + setsize*lag + objw*repet + objw*lag + repside*lag +
					   setsize*objw*lag)
model5d <- update(model4, logrt ~ setsize+objw+repet+repside+lag+
                                  setsize*objw + setsize*repside + setsize*lag + objw*repet + objw*lag + repside*lag +
					   setsize*repside*lag)
model5e <- update(model4, logrt ~ setsize+objw+repet+repside+lag+
                                  setsize*objw + setsize*repside + objw*repet + objw*repside + objw*lag + repside*lag +
					   objw*repside*lag)

anova(model4, model5a)
anova(model4, model5b)
anova(model4, model5c)
anova(model4, model5d)
anova(model4, model5e)

# 5d, 5e lead to improvement on all fit indices --> combine and add random effects

model6 <- update(model5e, logrt ~ setsize+objw+repet+repside+lag+
                                  setsize*objw + setsize*repside + setsize*lag + 
                                  objw*repet + objw*repside + objw*lag + repside*lag +
					    setsize*repside*lag + objw*repside*lag)
model6a <- update(model6, random = pdDiag(~ setsize+objw+repet+repside+lag+
                                  setsize*objw + setsize*repside + setsize*lag + 
                                  objw*repet + objw*repside + objw*lag + repside*lag +
					    setsize*repside*lag + objw*repside*lag))
anova(model6, model6a)
#model6 is superior in BIC

model6b <- update(model6, logrt ~ setsize+objw+repet+repside+lag+
                                  setsize*objw + setsize*repside + objw*repet + objw*repside + repside*lag +
					    setsize*objw*lag + setsize*repside*lag+objw*repside*lag + 
				          setsize*objw*repside*lag)
#this last one forces all 3-way interactions into the model - but it fits worse than 6. 


## now take best model so far and drop random effects (from back to front in the list); when main effects are dropped, all interactions including it must be dropped too!
model7a <- update(model6, random = pdDiag(~ setsize+objw+repet+repside+lag+
                                  objw*lag))
model7b <- update(model6, random = pdDiag(~ setsize+objw+repet+repside+lag+
                                  repside*lag))
model7c <- update(model6, random = pdDiag(~ setsize+objw+repet+repside +
                                  objw*lag))
model7d <- update(model6, random = pdDiag(~ setsize+objw+repet+lag))
model7e <- update(model6, random = pdDiag(~ setsize+objw+repside+lag+
                                  objw*lag+repside*lag))
model7f <- update(model6, random = pdDiag(~ setsize+repet+repside+lag+
                                  repside*lag))
model7g <- update(model6, random = pdDiag(~ objw+repet+repside+lag+
                                  objw*lag+repside*lag))


anova(model6, model7a)
anova(model6, model7b)
anova(model6, model7c)
anova(model6, model7d)
anova(model6, model7e)
anova(model6, model7f)
anova(model6, model7g)

# model 6 always comes out best


#removing setsize*objw does not impair fit
model8 <- update(model6, logrt ~ setsize+objw+repet+repside+lag+
                                  setsize*repside + setsize*lag + 
                                  objw*repet + objw*repside + objw*lag + repside*lag +
					    setsize*repside*lag + objw*repside*lag,
		    random = pdDiag(~ setsize+objw+repet+repside+lag+
                                  objw*lag + repside*lag))

#### Check upd as covariate 
model9 <- update (model8, logrt ~ setsize+objw+repet+repside+lag+upd+
                                  setsize*repside + setsize*lag + 
                                  objw*repet + objw*repside + objw*lag + repside*lag +
					    setsize*repside*lag + objw*repside*lag)
anova(model8, model9)
# significant improvement!


## finally, try to remove fixed effects to test their significance (from back to front in list of fixed effects, including the two-way interactions implied by the 3-way interactions). Also remove random effects associated with removed fixed effects

#remove objw*repside*lag
model9a <- update(model9, logrt ~ setsize+objw+repet+repside+lag+upd+
                                  setsize*repside + setsize*lag + 
                                  objw*repet + objw*repside + objw*lag + repside*lag +
					    setsize*repside*lag,
		    random = pdDiag(~ setsize+objw+repet+repside+lag+
                                  objw*lag + repside*lag))

#removing setsize*repside*lag
model9b <- update(model9, logrt ~ setsize+objw+repet+repside+lag+upd+
                                  setsize*repside + setsize*lag + 
                                  objw*repet + objw*repside + objw*lag + repside*lag +
					    objw*repside*lag,
		    random = pdDiag(~ setsize+objw+repet+repside+lag+
                                  objw*lag + repside*lag))

#removing repside*lag (plus higher order interactions)
model9c <- update(model9, logrt ~ setsize+objw+repet+repside+lag+upd+
                                  setsize*repside + setsize*lag + 
                                  objw*repet + objw*repside + objw*lag,
		    random = pdDiag(~ setsize+objw+repet+repside+lag+
                                  objw*lag))

#removing objw*lag (plus higher order interaction)
model9d <- update(model9, logrt ~ setsize+objw+repet+repside+lag+upd+
                                  setsize*repside + setsize*lag + 
                                  objw*repet + objw*repside + repside*lag +
					    setsize*repside*lag,
		    random = pdDiag(~ setsize+objw+repet+repside+lag+
                                  repside*lag))

#removing objw*repside (+ one higher order interaction)
model9e <- update(model9, logrt ~ setsize+objw+repet+repside+lag+upd+
                                  setsize*repside + setsize*lag + 
                                  objw*repet + objw*lag + repside*lag +
					    setsize*repside*lag,
		    random = pdDiag(~ setsize+objw+repet+repside+lag+
                                  objw*lag + repside*lag))

#removing objw*repet
model9f <- update(model9, logrt ~ setsize+objw+repet+repside+lag+upd+
                                  setsize*repside + setsize*lag + 
                                  objw*repside + objw*lag + repside*lag +
					    setsize*repside*lag + objw*repside*lag,
		    random = pdDiag(~ setsize+objw+repet+repside+lag+
                                  objw*lag + repside*lag))

#removing setsize*lag (and its interactions)
model9g <- update(model9, logrt ~ setsize+objw+repet+repside+lag+upd+
                                  setsize*repside + 
                                  objw*repet + objw*repside + objw*lag + repside*lag +
					    objw*repside*lag,
		    random = pdDiag(~ setsize+objw+repet+repside+lag+
                                  objw*lag + repside*lag))

#removing setsize*repside (+ 1 higher-order interaction)
model9h <- update(model9, logrt ~ setsize+objw+repet+repside+lag+upd+
                                  setsize*lag + 
                                  objw*repet + objw*repside + objw*lag + repside*lag +
					    objw*repside*lag,
		    random = pdDiag(~ setsize+objw+repet+repside+lag+
                                  objw*lag + repside*lag))

#removing upd
model9i <- update(model9, logrt ~ setsize+objw+repet+repside+lag+
                                  setsize*repside + setsize*lag + 
                                  objw*repet + objw*repside + objw*lag + repside*lag +
					    objw*repside*lag + setsize*repside*lag,
		    random = pdDiag(~ setsize+objw+repet+repside+lag+
                                  objw*lag + repside*lag))

#removing lag (+ all interactions it is involved in)
model9j <- update(model9, logrt ~ setsize+objw+repet+repside+upd+
                                  setsize*repside + objw*repet + objw*repside,
		    random = pdDiag(~ setsize+objw+repet+repside))

#removing repside (and all its interactions)
model9k <- update(model9, logrt ~ setsize+objw+repet+lag+upd+
                                  setsize*lag + objw*repet + objw*lag,
		    random = pdDiag(~ setsize+objw+repet+lag+objw*lag))

#removing repet (and all its interactions)
model9l <- update(model9, logrt ~ setsize+objw+repside+lag+upd+
                                  setsize*repside + setsize*lag + 
                                  objw*repside + objw*lag + repside*lag +
					    setsize*repside*lag + objw*repside*lag,
		    random = pdDiag(~ setsize+objw+repside+lag+
                                  objw*lag + repside*lag))

#removing objw (and all its interactions)
model9m <- update(model9, logrt ~ setsize+repet+repside+lag+upd+
                                  setsize*repside + setsize*lag + repside*lag +
					    setsize*repside*lag,
		    random = pdDiag(~ setsize+repet+repside+lag+ repside*lag))

#removing setsize and its interactions
model9n <- update(model9, logrt ~ objw+repet+repside+lag+upd+
                                  objw*repet + objw*repside + objw*lag + repside*lag +
					    objw*repside*lag,
		    random = pdDiag(~ objw+repet+repside+lag+
                                  objw*lag + repside*lag))


anova(model9, model9a)
anova(model9, model9b)
anova(model9, model9c)
anova(model9, model9d)
anova(model9, model9e)
anova(model9, model9f)
anova(model9, model9g)
anova(model9, model9h)
anova(model9, model9i)
anova(model9, model9j)
anova(model9, model9k)
anova(model9, model9l)
anova(model9, model9m)
anova(model9, model9n)


intervals(model9, which='fixed')


source("Rsquared.R")
R <- Rsquared(model9, gdata$logrt)
R

save(model9, file = "pi3.pos.model9.rda")



########################## Visualization #######################################

load("pi3.pos.model9.rda")

gdata$logpred <- predict(model9,level=1)
gdata$rt <- exp(gdata$logrt)
gdata$pred <- exp(gdata$logpred)
write.table(gdata, file = "pi3pos.predobs.dat", append = FALSE, quote = FALSE, sep = " ")
### writes out for SPSS --> aggregate by conditions and read again!


aggrdata=read.table("pi3pos.aggr.dat", header=TRUE, dec=".")
aggrdata$condition <- 10000*((aggrdata$setsize+1)*3-1) + 1000*(aggrdata$objw+1) + 100*(aggrdata$repet+1) +
                      10*(aggrdata$repside+2) + (aggrdata$lag+1)
#objw 1 = no, 2 = yes / repet: 1 = new, 2 = repeated / repside: 1 = same side, 2 = new, 3 = other side 
rdata <- reshape(aggrdata, idvar="vp", timevar="condition", 
         drop=c("setsize","objw","repet","repside","lag","logrt","logpred"), direction="wide")

rdata2 <- rdata[,1:37]
rdata5 <- rdata[,c(1,38:73)]

############# Now first deal wit setsize 2 ############

noswi2.new <- rdata2$rt.21121   #non-repeated items
noswi2.ss.obs <- rdata2[,c(4,6,8,10)]
noswi2.ss.pred <- rdata2[,c(5,7,9,11)]
noswi2.os.obs <- rdata2[,c(14,16,18)]
noswi2.os.pred <- rdata2[,c(15,17,19)]
swi2.new <- rdata2$rt.22121 
swi2.ss.obs <- rdata2[,c(22,24,26)]
swi2.ss.pred <- rdata2[,c(23,25,27)]
swi2.os.obs <- rdata2[,c(30,32,34,36)]
swi2.os.pred <- rdata2[,c(31,33,35,37)]

#Bakeman-McArthur correction
noswi2.mean <- rowMeans(cbind(noswi2.ss.obs, noswi2.os.obs), na.rm=T)
noswi2.ss.obs <- noswi2.ss.obs - noswi2.mean + mean(noswi2.mean, na.rm=T)
noswi2.os.obs <- noswi2.os.obs - noswi2.mean + mean(noswi2.mean, na.rm=T)
swi2.mean <- rowMeans(cbind(swi2.ss.obs, swi2.os.obs),na.rm=T)
swi2.ss.obs <- swi2.ss.obs - swi2.mean + mean(swi2.mean, na.rm=T)
swi2.os.obs <- swi2.os.obs - swi2.mean + mean(swi2.mean, na.rm=T)


source("plot.confint.R")
pt = c(1,0,17,0)
# point styles
par(font.main=1)
par(cex = 1.2)
#increases all text --> good for PowerPoint
cl <- c("black", "black", "black", "black")
#for print
#cl <- c("black", "red", "black", "blue")
#for PowerPoint
ltype <- c(1,1,2,2)
names <- c("1", "2", "3", "4+")

par(mfrow=c(2,1))
par(cex=0.7)
par(mar=c(4.5,4,3,3))

### no switch

x11()
plot.confint(noswi2.ss.obs, c(1:4), c(1:4), 0.05, xlim = c(0.5,4.5), ylim=c(0.5,2.5), cex=1.5,
             pch=pt[1], lty=ltype[1], xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot(c(1:4), colMeans(noswi2.ss.pred), xlim=c(0.5,4.5), ylim=c(0.5,2.5), type = "l", 
             pch=pt[2], lty=ltype[2], cex=2, lwd=3, xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot.confint(noswi2.os.obs, c(1:3), c(1,3,4), -0.05, xlim = c(0.5,4.5), ylim=c(0.5,2.5), cex=1.5,
             pch=pt[3], lty=ltype[3], xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot(c(1,3,4), colMeans(noswi2.os.pred), xlim=c(0.5,4.5), ylim=c(0.5,2.5), type = "l", 
             pch=pt[4], lty=ltype[4], cex=2, lwd=3, xlab="Lag", ylab="RT (s)", axes=F)

axis(side=1, at=c(1:4), labels=names)
axis(2) #brings back y-axis
box()   #brings back the frame
legend(0.5, 1.6, legend=c("same side", "other side"), 
        col=cl[c(1,3)], lty=ltype[c(1,3)], pch=pt[c(1,3)], xjust=0, yjust=0)
polygon(x=c(0,5), y=c(mean(noswi2.new),mean(noswi2.new)))
title(main="2 Positions, No Location Switch")

### location switch

x11()
plot.confint(na.omit(swi2.ss.obs), c(1:3), c(1,3,4), 0.05, xlim = c(0.5,4.5), ylim=c(0.5,2.5), cex=1.5,
             pch=pt[1], lty=ltype[1], xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot(c(1,3,4), colMeans(na.omit(swi2.ss.pred)), xlim=c(0.5,4.5), ylim=c(0.5,2.5), type = "l", 
             pch=pt[2], lty=ltype[2], cex=2, lwd=3, xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot.confint(na.omit(swi2.os.obs), c(1:4), c(1:4), -0.05, xlim = c(0.5,4.5), ylim=c(0.5,2.5), cex=1.5,
             pch=pt[3], lty=ltype[3], xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot(c(1:4), colMeans(na.omit(swi2.os.pred)), xlim=c(0.5,4.5), ylim=c(0.5,2.5), type = "l", 
             pch=pt[4], lty=ltype[4], cex=2, lwd=3, xlab="Lag", ylab="RT (s)", axes=F)

axis(side=1, at=c(1:4), labels=names)
axis(2) #brings back y-axis
box()   #brings back the frame
legend(0.5, 0.5, legend=c("same side", "other side"), 
        col=cl[c(1,3)], lty=ltype[c(1,3)], pch=pt[c(1,3)], xjust=0, yjust=0)
polygon(x=c(0,5), y=c(mean(swi2.new),mean(swi2.new)))
title(main="2 Positions, Location Switch")


############# Now deal wit setsize 3 ############

noswi5.new <- rdata5$rt.51121   #non-repeated items
noswi5.ss.obs <- rdata5[,c(4,6,8,10)]
noswi5.ss.pred <- rdata5[,c(5,7,9,11)]
noswi5.os.obs <- rdata5[,c(14,16,18)]
noswi5.os.pred <- rdata5[,c(15,17,19)]
swi5.new <- rdata5$rt.52121 
swi5.ss.obs <- rdata5[,c(22,24,26)]
swi5.ss.pred <- rdata5[,c(23,25,27)]
swi5.os.obs <- rdata5[,c(30,32,34,36)]
swi5.os.pred <- rdata5[,c(31,33,35,37)]

#Bakeman-McArthur correction
noswi5.mean <- rowMeans(cbind(noswi5.ss.obs, noswi5.os.obs), na.rm=T)
noswi5.ss.obs <- noswi5.ss.obs - noswi5.mean + mean(noswi5.mean, na.rm=T)
noswi5.os.obs <- noswi5.os.obs - noswi5.mean + mean(noswi5.mean, na.rm=T)
swi5.mean <- rowMeans(cbind(swi5.ss.obs, swi5.os.obs),na.rm=T)
swi5.ss.obs <- swi5.ss.obs - swi5.mean + mean(swi5.mean, na.rm=T)
swi5.os.obs <- swi5.os.obs - swi5.mean + mean(swi5.mean, na.rm=T)


source("plot.confint.R")
pt = c(1,0,17,0)
# point styles
par(font.main=1)
par(cex = 1.2)
#increases all text --> good for PowerPoint
cl <- c("black", "black", "black", "black")
#for print
#cl <- c("black", "red", "black", "blue")
#for PowerPoint
ltype <- c(1,1,2,2)
names <- c("1", "2", "3", "4+")

par(mfrow=c(2,1))
par(cex=0.7)
par(mar=c(4.5,4,3,3))

### no switch

x11()
plot.confint(noswi5.ss.obs, c(1:4), c(1:4), 0.05, xlim = c(0.5,4.5), ylim=c(0.5,4.0), cex=1.5,
             pch=pt[1], lty=ltype[1], xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot(c(1:4), colMeans(noswi5.ss.pred), xlim=c(0.5,4.5), ylim=c(0.5,4.0), type = "l", 
             pch=pt[2], lty=ltype[2], cex=2, lwd=3, xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot.confint(na.omit(noswi5.os.obs), c(1:3), c(1,3,4), -0.05, xlim = c(0.5,4.5), ylim=c(0.5,4.0), cex=1.5,
             pch=pt[3], lty=ltype[3], xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot(c(1,3,4), colMeans(na.omit(noswi5.os.pred)), xlim=c(0.5,4.5), ylim=c(0.5,4.0), type = "l", 
             pch=pt[4], lty=ltype[4], cex=2, lwd=3, xlab="Lag", ylab="RT (s)", axes=F)

axis(side=1, at=c(1:4), labels=names)
axis(2) #brings back y-axis
box()   #brings back the frame
legend(0.5, 3.1, legend=c("same location", "other location"), 
        col=cl[c(1,3)], lty=ltype[c(1,3)], pch=pt[c(1,3)], xjust=0, yjust=0)
polygon(x=c(0,5), y=c(mean(noswi5.new),mean(noswi5.new)))
title(main="3 Positions, No Location Switch")

### location switch

x11()
plot.confint(na.omit(swi5.ss.obs), c(1:3), c(1,3,4), 0.05, xlim = c(0.5,4.5), ylim=c(0.5,4.0), cex=1.5,
             pch=pt[1], lty=ltype[1], xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot(c(1,3,4), colMeans(na.omit(swi5.ss.pred)), xlim=c(0.5,4.5), ylim=c(0.5,4.0), type = "l", 
             pch=pt[2], lty=ltype[2], cex=2, lwd=3, xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot.confint(na.omit(swi5.os.obs), c(1:4), c(1:4), -0.05, xlim = c(0.5,4.5), ylim=c(0.5,4.0), cex=1.5,
             pch=pt[3], lty=ltype[3], xlab="Lag", ylab="RT (s)", axes=F)
par(new=T)
plot(c(1:4), colMeans(na.omit(swi5.os.pred)), xlim=c(0.5,4.5), ylim=c(0.5,4.0), type = "l", 
             pch=pt[4], lty=ltype[4], cex=2, lwd=3, xlab="Lag", ylab="RT (s)", axes=F)

axis(side=1, at=c(1:4), labels=names)
axis(2) #brings back y-axis
box()   #brings back the frame
legend(2.8, 0.7, legend=c("same location", "other location"), 
        col=cl[c(1,3)], lty=ltype[c(1,3)], pch=pt[c(1,3)], xjust=0, yjust=0)
polygon(x=c(0,5), y=c(mean(swi5.new),mean(swi5.new)))
title(main="3 Positions, Location Switch")




